const mongoose = require('mongoose');

const Campaign = mongoose.model('Campaign');
const Order = mongoose.model('Order');
const Message = mongoose.model('Message');
const Subscriber = mongoose.model('Subscriber');
const Merchant = mongoose.model('Merchant');
const Verify = mongoose.model('Verify');

const { decodeToken } = require('../helpers/jwt.helper');
const { getValidPhoneNumber } = require('../helpers/phoneNumber.helper');
const {
    sendVerificationCodeMessage,
    sendCampaignPreviewMessage
} = require('../helpers/message.helper');

const {
    SUBSCRIBER_SUBSCIRBED_STATUS,
    PHONE_NUMBER_VERIFIED_STATUS,
    CAMPAIGN_SCHEDULED_STATUS
} = require('../helpers/status.helper');

const list = async (req, res) => {
    const token = req.get("X-JWT-Token");
    const decodedToken = decodeToken(token);

    if (!decodedToken) return res.sendStatus(503);

    const { merchantId } = decodedToken.data;

    const campaigns = await Campaign
        .find({ merchant: merchantId })
        .sort({ createdAt: -1 });

    return res.status(200).json({ campaigns });
}

const create = async (req, res) => {
    const token = req.get("X-JWT-Token");
    const decodedToken = decodeToken(token);

    if (!decodedToken) return res.sendStatus(503);

    const { title, date, message, price } = req.body;

    const media = req.file ? `${req.file.key}` : null;

    const { merchantId } = decodedToken.data;

    const recepients = Subscriber.find({
        status: SUBSCRIBER_SUBSCIRBED_STATUS,
        merchant: merchantId,
        isDeleted:0
    });

    const campaign = new Campaign({
        title,
        date,
        status: CAMPAIGN_SCHEDULED_STATUS,
        recepients,
        media,
        message,
        price,
        merchant: merchantId
    });

    await campaign.save();

    return res.sendStatus(200);
};

const view = async (req, res) => {
    const token = req.get("X-JWT-Token");
    const decodedToken = decodeToken(token);

    if (!decodedToken) return res.sendStatus(503);

    const { merchantId } = decodedToken.data;
    const { campaignId } = req.params;

    const campaign = await Campaign.findOne({ merchant: merchantId, _id: campaignId });
    const orders = await Order
        .find({ merchant: merchantId, campaign: campaign._id })
        .sort({ createdAt: -1 });
        const messages = await Message
        .find({ merchant: merchantId, campaign: campaign._id })
        .sort({ createdAt: -1 });

    return res.status(200).json({ campaign, orders, messages });
};

const verify = async (req, res) => {
    const token = req.get("X-JWT-Token");
    const decodedToken = decodeToken(token);

    if (!decodedToken) return res.sendStatus(503);

    const merchant = await Merchant.findById(decodedToken.data.merchantId);

    const { phoneNumber } = req.query;

    const scrubbedNumber = await getValidPhoneNumber(phoneNumber);
    if (!scrubbedNumber.phoneNumber) return res.sendStatus(503);

    await sendVerificationCodeMessage(merchant, scrubbedNumber.phoneNumber);

    return res.sendStatus(200);
}

const verifyCode = async (req, res) => {
    const { verificationCode } = req.params;

    const verified = await Verify.findOne({ code: verificationCode });

    return res.status(200).json({
        status: verified ?
            PHONE_NUMBER_VERIFIED_STATUS
            :
            PHONE_NUMBER_VERIFICATION_FAILED_STATUS
    });
};

const preview = async (req, res) => {
    const token = req.get("X-JWT-Token");
    const decodedToken = decodeToken(token);

    //if (!decodedToken) return res.sendStatus(503);

    const merchant = await Merchant.findById(decodedToken.data.merchantId);

    const { phoneNumber, message } = req.body;

    //const scrubbedNumber = await getValidPhoneNumber(phoneNumber.phoneNumber);

    await sendCampaignPreviewMessage(merchant, phoneNumber, message);

    return res.sendStatus(200);
};

module.exports = {
    list,
    create,
    view,
    verify,
    verifyCode,
    preview
};