const mongoose = require('mongoose');
const Stripe = require('stripe');
require('dotenv').config();

const Merchant = mongoose.model('Merchant');
const Order = mongoose.model('Order');
const Subscriber = mongoose.model('Subscriber');
const Message = mongoose.model('Message');
const  Wallet= mongoose.model("Wallet");
const stripe =Stripe(process.env.STRIPE_SECRET_KEY);

const {
    checkForAvailableLongcodes,
    provisionNewLongcodeFromTwilio,
    getTwilioAllNumbers
} = require('../helpers/longcode.helper');

const {
    hashPassword,
    isValidPassword,
    generateSlug
} = require('../helpers/merchant.helper');

const {sendMail} = require('../helpers/mail.helper');
const { generateToken, decodeToken } = require('../helpers/jwt.helper');

const verify = async (req, res) => {
    const token = req.get("X-JWT-Token");
    const decodedToken = decodeToken(token);
    if (!decodedToken) return res.sendStatus(503);
    const merchant = await Merchant.findById(decodedToken.data.merchantId);
    const date = new Date();
    // const trialExpired = merchant.plan === 'trial'|| 'paid' && (new Date(merchant.trial.endsAt) <= date);
    const trialExpired =(new Date(merchant.trial.endsAt) <= date);
    return res.status(200).json({ trialExpired });
}

const create = async (req, res) => {
    const {
        email,
        insecurePassword,
        firstName,
        lastName,
        phoneNumber,
        name
    } = req.body;

    const emailAlreadyRegistered = await Merchant.findOne({ email });
    if (emailAlreadyRegistered) return res.sendStatus(500);

    const password = await hashPassword(insecurePassword);
    const account = await stripe.accounts.create({
        type: 'standard',
        email,
        business_type: 'company',
        company: {
            phone: phoneNumber
        }
    });


    const customer = await stripe.customers.create({
        email,
        name: `${firstName} ${lastName}`
    });
   
    const owner = { firstName, lastName };
    const date = new Date();
    const endDate = new Date();
    endDate.setDate(date.getDate() + 14);

    // const longcodeAvailable = await checkForAvailableLongcodes();
    // const longcode = longcodeAvailable? longcodeAvailable.phoneNumber : await provisionNewLongcodeFromTwilio();
     
    // get random phone number from twilio 
     const longcode = await getTwilioAllNumbers();
     console.log("twilio incoming phone number",longcode)

    const merchant = new Merchant({
        email,
        password,
        owner,
        phoneNumber,
        name,
        longcode,
        subdomain: await generateSlug(name),
        stripe: {
            customer:{id: customer.id},
            account: { id: account.id }
        },
        trial: {
            startsAt: date,
            endsAt: endDate
        }
    });

    console.log("merchant create",merchant)
    
    await merchant.save();
    
    // default wallet with 20$
    const wallet = new Wallet({
        stripe:{
          customer:{
            id:merchant.stripe.customer.id
          }
        },
        merchant:merchant._id,
        charged_amount:20,
        amount:20  
   });
     await wallet.save();

    const payload = { merchantId: merchant._id, name, square: false };
    const token = generateToken(payload);
    return res.status(200).json({ token });
};

const login = async (req, res) => {
    const { email, password } = req.body;
    if (!email || !password) return res.sendStatus(503);

    const merchant = await Merchant.findOne({ email });
    if (!merchant) return res.status(400).send({message:" Sorry Email not exist !"});

    const validPassword = await isValidPassword(password, merchant.password);
    if (!validPassword) return res.status(400).send({message:"Incorrect password !"});

    const payload = { merchantId: merchant._id, name: merchant.name, square: false };
    const token = generateToken(payload);

    return res.status(200).json({ token });
};

const dashboard = async (req, res) => {
    const token = req.get("X-JWT-Token");
    const decodedToken = decodeToken(token);
    if (!decodedToken) return res.sendStatus(503);

    const merchant = await Merchant.findById(decodedToken.data.merchantId);

    const bankConnected = merchant.stripe.account.connected;

    const orders = await Order.find({
        merchant: decodedToken.data.merchantId
    }, 'total date');

    const subscribers = await Subscriber.find({
        merchant: decodedToken.data.merchantId
    }, 'createdAt');

    const messages = await Message.find({
        merchant: decodedToken.data.merchantId
    }, 'createdAt');

    return res.status(200).json({
        bankConnected,
        orders,
        subscribers,
        messages,
        merchant
    });
};

const loadLandingPageContent = async (req, res) => {
    const { subdomain } = req.params;
    const merchant = await Merchant.findOne({ subdomain });

    return res.status(200).json({
        id: merchant._id,
        name: merchant.name,
        headline: merchant.headline,
        description: merchant.description,
        youtubeVideo: merchant.youtubeVideo,
        logo: merchant.logo,
        stripeAccount: merchant.stripe.account.id,
        fontFamily: merchant.fontFamily,
        fbPixel: merchant.fbPixel,
        ageWarning: merchant.ageWarning
    });
};

const embedSnippet = async (req, res) => {
    const token = req.get("X-JWT-Token");
    const decodedToken = decodeToken(token);
    if (!decodedToken) return res.sendStatus(503);

    const merchant = await Merchant.findById(decodedToken.data.merchantId);

    return res.status(200).json({ subdomain: merchant.subdomain });
};

const forgotPassword = async(req,res)=>{
    try{
        const {email,password , confirm_password}= req.body ;
        if(!email || !password || !confirm_password) return res.status(201).send({message: 'All fields are required !'});
        if(password !== confirm_password) return res.status(201).send({message:"Password and confirm password are not matched"});
        const merchant = await Merchant.findOne({email});
        if(!merchant) {
            res.status(201).send({message:"Email not exist !"})
        } else{
            merchant.password = await hashPassword(confirm_password)
            merchant.save()
            res.status(200).send({
                message:"password change succesfully !"
            })
        }
            
       

    }catch(error){
        res.status(400).send({
            message:"Oops ! something went wrong in forgot password",
            subError:error.message
        })
    }
} 

const sendforgetLink = async(req,res) =>{
    try{
        const {email} = req.body ;
        if(!email){
            res.status(201).send({
                message:"email is required !"
            })
        }else{
          const mailsend = await sendMail(email);
         
           if(mailsend){
                res.status(200).send({
                    message:'Mail Sent Successfully ! Please Check Your Email'
                })
           }
        }
    }catch(error){
        res.status(400).send({
            message:"Oops! something went wrong in send forgot pasword link",
            subError:error.message
        })
    }
}
module.exports = {
    create,
    login,
    verify,
    dashboard,
    loadLandingPageContent,
    embedSnippet,
    forgotPassword,
    sendforgetLink
};