const mongoose = require('mongoose');
const Stripe = require('stripe');

require('dotenv').config();

const Merchant = mongoose.model('Merchant');

const stripe = Stripe(process.env.STRIPE_SECRET_KEY);

const { decodeToken } = require('../helpers/jwt.helper');

const getAccountLink = async (req, res) => {
    const token = req.get("X-JWT-Token");
    const decodedToken = decodeToken(token);

    if (!decodedToken) return res.sendStatus(503);

    const merchant = await Merchant.findById(decodedToken.data.merchantId);

    if (merchant.stripe.account.connected)
        return res.status(200).json({ connected: true });

    const accountLinks = await stripe.accountLinks.create({
        account: merchant.stripe.account.id,
        refresh_url: `${process.env.CLIENT_URL}/stripe/reauth?state=${req.query.state}`,
        return_url: `${process.env.CLIENT_URL}/stripe/callback?state=${req.query.state}`,
        type: 'account_onboarding',
    });

    merchant.stripe.account.temporaryState = req.query.state;
    await merchant.save();

    return res.status(200).json({
        link: accountLinks.url,
        emailRequired: merchant.stripe.account.emailRequired
    });
};

const reauth = async (req, res) => {
    const token = req.get("X-JWT-Token");
    const decodedToken = decodeToken(token);

    if (!decodedToken) return res.sendStatus(503);

    const merchant = await Merchant.findById(decodedToken.data.merchantId);

    const accountLinks = await stripe.accountLinks.create({
        account: merchant.stripe.account.id,
        refresh_url: `${process.env.CLIENT_URL}/stripe/reauth?state=${req.query.temporaryState}`,
        return_url: `${process.env.CLIENT_URL}/stripe/callback?state=${req.query.temporaryState}`,
        type: 'account_onboarding',
    });

    merchant.stripe.account.temporaryState = req.query.temporaryState;
    await merchant.save();

    return res.status(200).json({ url: accountLinks.url });
};

const callback = async (req, res) => {
    const token = req.get("X-JWT-Token");
    const decodedToken = decodeToken(token);

    if (!decodedToken) return res.sendStatus(503);

    const merchant = await Merchant.findById(decodedToken.data.merchantId);
    const account = await stripe.accounts.retrieve(merchant.stripe.account.id);

    if (account.charges_enabled) {
        merchant.stripe.account.temporaryState = "";
        merchant.stripe.account.connected = true;
        await merchant.save();
    }
    else {
        merchant.stripe.account.temporaryState = "";
        merchant.stripe.account.emailRequired = true;
        await merchant.save();
    }

    return res.sendStatus(200);
};

const setupIntents = async(req,res)=>{
    try{
        const {customer , stripeAccount} = req.body ;
        if(!customer || !stripeAccount){
            res.status(201).send({
                message:'customer id or stripe account not found !'
            });
            return
        }
        const intent = await stripe.setupIntents.create(
            {
              customer: customer,
            },
            {
              stripeAccount: stripeAccount,
            } 
          );
          
          res.status(200).send({
            clientSecret:intent.client_secret
          });
    }catch(error){
        res.status(400).send({
            message:error.message
        })
    }
}
module.exports = { getAccountLink, reauth, callback ,setupIntents};