const mongoose = require("mongoose");
const Stripe = require("stripe");

require("dotenv").config();

const Subscriber = mongoose.model("Subscriber");
const Order = mongoose.model("Order");
const Merchant = mongoose.model("Merchant");
const  Wallet= mongoose.model("Wallet");

const stripe = Stripe(process.env.STRIPE_SECRET_KEY);

const {
  sendNewSubscriberMessage,
  sendWelcomeMessage,
} = require("../helpers/message.helper");

const { getValidPhoneNumber } = require("../helpers/phoneNumber.helper");
const { decodeToken } = require("../helpers/jwt.helper");

const {SUBSCRIBER_BY_CSV , SUBSCRIBER_BY_IFRAME,SUBSCRIBER_SUBSCIRBED_STATUS ,SUBSCRIBER_PENDING_SUBSCIRBED_STATUS } = require("../helpers/status.helper");

const {
  config,
  createSubscription,
} = require("../helpers/stripe.subscriber.helper");

const list = async (req, res) => {
  const token = req.get("X-JWT-Token");
  const decodedToken = decodeToken(token);

  if (!decodedToken) return res.sendStatus(503);

  const subscribers = await Subscriber.find({
    merchant: decodedToken.data.merchantId,
    isDeleted:0
  }).sort({ createdAt: -1 });

  return res.status(200).json({ subscribers });
};

const create = async (req, res) => {
  const {
    phoneNumber,
    firstName,
    lastName,
    addressOne,
    addressTwo,
    businessName,
    zipCode,
    city,
    state,
    merchant,
  } = req.body;

  const validatedNumber = await getValidPhoneNumber(phoneNumber);

  const doesSubscriberExist = await Subscriber.findOne({
    phoneNumber: validatedNumber.phoneNumber,
    merchant,
    isDeleted:0
  });
  const merchantToUse = await Merchant.findById(merchant);

  if (
    !phoneNumber ||
    !firstName ||
    !lastName ||
    !addressOne ||
    !zipCode ||
    !city ||
    !state ||
    !merchant ||
    !validatedNumber.phoneNumber ||
    doesSubscriberExist
  ){
    res.status(201).send({
      message:"already subscriber exist or number not valid"
    })
  }else if (!merchantToUse) {
    res.status(201).send({
      message:"merchant not found !"
    })
  }else{
    const subscriber = new Subscriber({
      nationalFormat: validatedNumber.nationalFormat,
      phoneNumber: validatedNumber.phoneNumber,
      status: SUBSCRIBER_SUBSCIRBED_STATUS,
      firstName,
      lastName,
      businessName,
      addressOne,
      addressTwo,
      zipCode,
      city,
      state,
      merchant,
      type:SUBSCRIBER_BY_IFRAME || 'iframe'
    });
  
    await subscriber.save();
  
    const customer = await stripe.customers.create(
      {
        address: {
          city,
          country: "US",
          line1: addressOne,
          line2: addressTwo,
          postal_code: zipCode,
          state,
        },
        description: `Signed up from ${merchant}`,
        name: businessName || `${firstName} ${lastName}`,
        phone: validatedNumber.phoneNumber,
        shipping: {
          address: {
            city,
            country: "US",
            line1: addressOne,
            line2: addressTwo,
            postal_code: zipCode,
            state,
          },
          name: `${firstName} ${lastName}`,
          phone: validatedNumber.phoneNumber,
        },
      },
      {
        stripeAccount: merchantToUse.stripe.account.id,
      }
    );
    console.log("customer ==>", customer);
  
    subscriber.stripe.customer.id = customer.id;
    await subscriber.save();
  
    const intent = await stripe.setupIntents.create(
      {
        customer: customer.id,
      },
      {
        stripeAccount: merchantToUse.stripe.account.id,
      }
    );
    await sendWelcomeMessage(subscriber,false);
    // await sendNewSubscriberMessage(subscriber, merchantToUse);
    res.status(200).send({
      subscriber: subscriber.id,
      clientSecret: intent.client_secret,
    });
  }

};

const setPaymentMethod = async (req, res) => {
  const { subscriberId } = req.params;
  const { paymentMethod } = req.query;

  const subscriber = await Subscriber.findById({_id:subscriberId,isDeleted:0});
  
  subscriber.stripe.customer.paymentMethod = paymentMethod;
  subscriber.status = SUBSCRIBER_SUBSCIRBED_STATUS
  await subscriber.save();

  return res.sendStatus(200);
};

const view = async (req, res) => {
  const { subscriberId } = req.params;

  const subscriber = await Subscriber.findById({_id:subscriberId,isDeleted:0});
  const orders = await Order.find({ subscriber: subscriber._id }).sort({
    createdAt: -1,
  });

  return res.status(200).json({
    subscriber,
    orders,
  });
};

const configsubscriber = async (req, res) => {
  const data = await config(req, res);
  res.status(200).send(data);
};

const subscribe129 = async (req, res) => {
  const {
    phoneNumber,
    firstName,
    lastName,
    addressOne,
    addressTwo,
    businessName,
    zipCode,
    city, 
    state,
    merchant,
  } = req.body.data;
  const validatedNumber = await getValidPhoneNumber(phoneNumber);
  const doesSubscriberExist = await Subscriber.findOne({
    phoneNumber: validatedNumber.phoneNumber,
    merchant,
    isDeleted:0
  });
  const data = await createSubscription(req, res);
 if (data && doesSubscriberExist) {
    const merchantToUse = await Merchant.findById(merchant);
    if (!merchantToUse) {
      res.status(500).send({
        message: "merchant not found",
      });
      return 
    }
    await sendWelcomeMessage(doesSubscriberExist,false);
    // await sendNewSubscriberMessage(doesSubscriberExist, merchantToUse);
    var today = new Date();
    var priorDate = new Date(new Date().setDate(today.getDate() + 30));
    const updateMerchant = await Merchant.findByIdAndUpdate(
      merchant,
      {
        trial: {
          startsAt: today,
          endsAt: priorDate,
        },
        plan:'paid'
      },
      { new: true }
    );
    if (updateMerchant) {
      res.status(200).send(data);
    }
  }

  if (data && !doesSubscriberExist) {
    if (
      !phoneNumber ||
      !firstName ||
      !lastName ||
      !addressOne ||
      !zipCode ||
      !city ||
      !state ||
      !merchant ||
      !validatedNumber.phoneNumber ||
      doesSubscriberExist
    ){
      res.status(500).send({
        message: "required data missing !",
      });
      return 
    }
    

    const merchantToUse = await Merchant.findById(merchant);
    if (!merchantToUse) {
      res.status(500).send({
        message: "merchant not found",
      });
      return 
    }
    const subscriber = new Subscriber({
      nationalFormat: validatedNumber.nationalFormat,
      phoneNumber: validatedNumber.phoneNumber,
      status: SUBSCRIBER_SUBSCIRBED_STATUS,
      firstName,
      lastName,
      businessName,
      addressOne,
      addressTwo,
      zipCode,
      city,
      state,
      merchant,
      type:SUBSCRIBER_BY_IFRAME || 'iframe'
    });

    await subscriber.save();
    subscriber.stripe.customer.id = merchantToUse.stripe.customer.id;
    await subscriber.save();
    await sendWelcomeMessage(subscriber,false);
    // await sendNewSubscriberMessage(subscriber, merchantToUse);
    var today = new Date();
    var priorDate = new Date(new Date().setDate(today.getDate() + 30));
    const updateMerchant = await Merchant.findByIdAndUpdate(
      merchant,
      {
        trial: {
          startsAt: today,
          endsAt: priorDate,
        },
        plan:'paid'
      },
      { new: true }
    );
    if (updateMerchant) {
      res.status(200).send(data);
    }
  }
};


const deleteSubscriber = async(req,res)=>{
  try{
    const { subscriberId } = req.params;
    
    if(!subscriberId){
      res.status(201).send({
        message:'please provide subscriber id !'
      })
    }else{
      const subscriber = await Subscriber.findById(subscriberId);
      if(!subscriber){
        res.status(201).send({
          message:'subscriber not found !'
        })
      }else{
        await Subscriber.deleteOne({ _id: subscriber._id })
        res.status(200).send({
          message:'Subscriber Deleted Successfully !'
        })
      }
    }
  }catch(error){
    res.status(400).send({
      message:'Oops ! someting went wrong in delete subscriber',
      subError:error.message
    })
  }
}


const createSubscriberByCsv = async (req, res) => {
   console.log("req.body => ",req.body)
   const subscriberImport = [];
   const subscriberToImport = req.body;
   subscriberToImport.forEach(async subscriber => {
     const {
    phoneNumber,
    firstName,
    lastName,
    addressOne,
    addressTwo,
    businessName,
    zipCode,
    city,
    state,
    merchant,
  } = subscriber;

  const validatedNumber = await getValidPhoneNumber(phoneNumber);

  const doesSubscriberExist = await Subscriber.findOne({
    // phoneNumber: validatedNumber.phoneNumber,
    merchant,
    isDeleted:0
  });
  const merchantToUse = await Merchant.findById(merchant);
 if (
    !phoneNumber ||
    !firstName ||
    !lastName ||
    !addressOne ||
    !zipCode ||
    !city ||
    !state ||
    !merchant ||
    !validatedNumber.phoneNumber ||
    doesSubscriberExist
  ){
    res.status(201).send({
      message:"already subscriber exist or number not valid by twilio"
    })
    return
  }else if (!merchantToUse) {
    res.status(201).send({
      message:"merchant not found !"
    })
    return
  }else{
    const subscriber = new Subscriber({
      nationalFormat: validatedNumber.nationalFormat,
      phoneNumber: validatedNumber.phoneNumber,
      status: SUBSCRIBER_PENDING_SUBSCIRBED_STATUS || 'pending',
      firstName,
      lastName,
      businessName,
      addressOne,
      addressTwo,
      zipCode,
      city,
      state,
      merchant,
      type:SUBSCRIBER_BY_CSV || 'import' 
    });
  
    await subscriber.save();
  
    const customer = await stripe.customers.create(
      {
        address: {
          city,
          country: "US",
          line1: addressOne,
          line2: addressTwo,
          postal_code: zipCode,
          state,
        },
        description: `Signed up from ${merchant}`,
        name: businessName || `${firstName} ${lastName}`,
        phone: validatedNumber.phoneNumber,
        shipping: {
          address: {
            city,
            country: "US",
            line1: addressOne,
            line2: addressTwo,
            postal_code: zipCode,
            state,
          },
          name: `${firstName} ${lastName}`,
          phone: validatedNumber.phoneNumber,
        },
      },
      {
        stripeAccount: merchantToUse.stripe.account.id,
      }
    );
    subscriber.stripe.customer.id = customer.id;
    await subscriber.save();
   // await sendWelcomeMessage(subscriber,true);
    res.status(200).send({
      message:'subscriber are imported !'
    });
  }
 
});


};


const onlyMarketingSubscriber = async(req,res)=>{
  try{
        const body = req.body;
        if( !body.merchant ||  !body.phoneNumber || !body.firstName || !body.lastName){
          res.status(201).send({message:'merchant/phoneNumber/firstName/lastName is required !'})
          return
        }

        const merchantToUse = await Merchant.findById(body.merchant);
        if(!merchantToUse){
          res.status(201).send({message:'merchant not exist !'});
          return
        }
        const phoneNumber = body.phoneNumber;
        const validatedNumber = await getValidPhoneNumber(phoneNumber);
        if(!validatedNumber){
          res.status(201).send({message:'Phone Number is not valid !'});
          return
        }
        const doesSubscriberExist = await Subscriber.findOne({
          phoneNumber: validatedNumber.phoneNumber,
          merchant: body.merchant,
          isDeleted:0
        });

    
        if(doesSubscriberExist){
            res.status(201).send({message:'subscriber already exist with this phone number'});
            return 
          }

        const subscriber = new Subscriber({
          nationalFormat: validatedNumber.nationalFormat,
          phoneNumber: validatedNumber.phoneNumber,
          status: SUBSCRIBER_PENDING_SUBSCIRBED_STATUS || 'pending',
          firstName:body.firstName,
          lastName:body.lastName,
          merchant:body.merchant,
          type:'generaliframe' 
        });
      
        await subscriber.save();
        await sendWelcomeMessage(subscriber,false);

        if(subscriber){
          res.status(200).send({message:'Subscriber created succesfully !'});
          return
        }
  }catch(error){
    res.status(400).send({
      message:'Oops ! something went wrong to create marketing subscriber',
      subError:error.message
    })
  }
};

module.exports = {
  list,
  create,
  setPaymentMethod,
  view,
  configsubscriber,
  subscribe129,
  deleteSubscriber,
  createSubscriberByCsv,
  onlyMarketingSubscriber
};
