import { Schema } from '../schema';
import { InvoicePaymentReminder } from './invoicePaymentReminder';
import { Money } from './money';
/**
 * Represents a payment request for an [invoice](#type-Invoice). Invoices can specify a maximum
 * of 13 payment requests, with up to 12 `INSTALLMENT` request types.
 * For more information,
 * see [Payment requests](https://developer.squareup.com/docs/invoices-api/overview#payment-requests).
 */
export interface InvoicePaymentRequest {
    /** The Square-generated ID of the payment request in an [invoice](#type-invoice). */
    uid?: string;
    /**
     * Specifies the action for Square to take for processing the invoice. For example,
     * email the invoice, charge a customer's card on file, or do nothing. DEPRECATED at version 2021-01-21. The corresponding `request_method` field is replaced by the `Invoice.delivery_method` and `InvoicePaymentRequest.automatic_payment_source` fields.
     */
    requestMethod?: string;
    /**
     * Indicates the type of the payment request. An invoice supports the following payment request combinations:
     * - 1 balance
     * - 1 deposit with 1 balance
     * - 2 - 12 installments
     * - 1 deposit with 2 - 12 installments
     * For more information,
     * see [Payment requests](https://developer.squareup.com/docs/invoices-api/overview#payment-requests).
     */
    requestType?: string;
    /**
     * The due date (in the invoice location's time zone) for the payment request, in `YYYY-MM-DD` format.
     * After this date, the invoice becomes overdue. This field is required to create a payment request.
     */
    dueDate?: string;
    /**
     * Represents an amount of money. `Money` fields can be signed or unsigned.
     * Fields that do not explicitly define whether they are signed or unsigned are
     * considered unsigned and can only hold positive amounts. For signed fields, the
     * sign of the value indicates the purpose of the money transfer. See
     * [Working with Monetary Amounts](https://developer.squareup.com/docs/build-basics/working-with-monetary-amounts)
     * for more information.
     */
    fixedAmountRequestedMoney?: Money;
    /**
     * Specifies the amount for the payment request in percentage:
     * - When the payment `request_type` is `DEPOSIT`, it is the percentage of the order total amount.
     * - When the payment `request_type` is `INSTALLMENT`, it is the percentage of the order total less
     * the deposit, if requested. The sum of the `percentage_requested` in all installment
     * payment requests must be equal to 100.
     * You cannot specify this when the payment `request_type` is `BALANCE` or when the
     * payment request specifies the `fixed_amount_requested_money` field.
     */
    percentageRequested?: string;
    /**
     * If set to true, the Square-hosted invoice page (the `public_url` field of the invoice)
     * provides a place for the customer to pay a tip.
     * This field is allowed only on the final payment request
     * and the payment `request_type` must be `BALANCE` or `INSTALLMENT`.
     */
    tippingEnabled?: boolean;
    /** Indicates the automatic payment method for an [invoice payment request](#type-InvoicePaymentRequest). */
    automaticPaymentSource?: string;
    /**
     * The ID of the card on file to charge for the payment request. To get the customer’s card on file,
     * use the `customer_id` of the invoice recipient to call [RetrieveCustomer](#endpoint-Customers-RetrieveCustomer)
     * in the Customers API. Then, get the ID of the target card from the `cards` field in the response.
     */
    cardId?: string;
    /** A list of one or more reminders to send for the payment request. */
    reminders?: InvoicePaymentReminder[];
    /**
     * Represents an amount of money. `Money` fields can be signed or unsigned.
     * Fields that do not explicitly define whether they are signed or unsigned are
     * considered unsigned and can only hold positive amounts. For signed fields, the
     * sign of the value indicates the purpose of the money transfer. See
     * [Working with Monetary Amounts](https://developer.squareup.com/docs/build-basics/working-with-monetary-amounts)
     * for more information.
     */
    computedAmountMoney?: Money;
    /**
     * Represents an amount of money. `Money` fields can be signed or unsigned.
     * Fields that do not explicitly define whether they are signed or unsigned are
     * considered unsigned and can only hold positive amounts. For signed fields, the
     * sign of the value indicates the purpose of the money transfer. See
     * [Working with Monetary Amounts](https://developer.squareup.com/docs/build-basics/working-with-monetary-amounts)
     * for more information.
     */
    totalCompletedAmountMoney?: Money;
    /**
     * Represents an amount of money. `Money` fields can be signed or unsigned.
     * Fields that do not explicitly define whether they are signed or unsigned are
     * considered unsigned and can only hold positive amounts. For signed fields, the
     * sign of the value indicates the purpose of the money transfer. See
     * [Working with Monetary Amounts](https://developer.squareup.com/docs/build-basics/working-with-monetary-amounts)
     * for more information.
     */
    roundingAdjustmentIncludedMoney?: Money;
}
export declare const invoicePaymentRequestSchema: Schema<InvoicePaymentRequest>;
